@@ -1,18 +1,19 @@
 /*
  * %kadu copyright begin%
- * Copyright 2007, 2008, 2009 Dawid Stawiarski (neeo@kadu.net)
- * Copyright 2010, 2011 Bartosz Brachaczek (b.brachaczek@gmail.com)
+ * Copyright 2010 Tomasz Rostanski (rozteck@interia.pl)
+ * Copyright 2008, 2009, 2010, 2010, 2011 Piotr Galiszewski (piotr.galiszewski@kadu.im)
  * Copyright 2009 Wojciech Treter (juzefwt@gmail.com)
- * Copyright 2008, 2009, 2010, 2011 Piotr Galiszewski (piotr.galiszewski@kadu.im)
- * Copyright 2004, 2005, 2006, 2007 Marcin Ślusarz (joi@kadu.net)
+ * Copyright 2008, 2009, 2010, 2010, 2010, 2010 Tomasz Rostański (rozteck@interia.pl)
+ * Copyright 2011 Piotr Dąbrowski (ultr@ultr.pl)
+ * Copyright 2004, 2007, 2008, 2009 Michał Podsiadlik (michal@kadu.net)
+ * Copyright 2010 Bartłomiej Zimoń (uzi18@o2.pl)
  * Copyright 2002, 2003, 2004, 2005, 2007 Adrian Smarzewski (adrian@kadu.net)
+ * Copyright 2004, 2005 Paweł Płuciennik (pawel_p@kadu.net)
  * Copyright 2002, 2003 Tomasz Chiliński (chilek@chilan.com)
  * Copyright 2007, 2008, 2009, 2010, 2011 Rafał Malinowski (rafal.przemyslaw.malinowski@gmail.com)
- * Copyright 2004, 2007, 2008, 2009 Michał Podsiadlik (michal@kadu.net)
- * Copyright 2010 Tomasz Rostanski (rozteck@interia.pl)
- * Copyright 2008, 2009, 2010 Tomasz Rostański (rozteck@interia.pl)
- * Copyright 2010 Bartłomiej Zimoń (uzi18@o2.pl)
- * Copyright 2004, 2005 Paweł Płuciennik (pawel_p@kadu.net)
+ * Copyright 2010, 2011 Bartosz Brachaczek (b.brachaczek@gmail.com)
+ * Copyright 2007, 2008, 2009 Dawid Stawiarski (neeo@kadu.net)
+ * Copyright 2004, 2005, 2006, 2007 Marcin Ślusarz (joi@kadu.net)
  * Copyright 2002, 2003 Dariusz Jagodzik (mast3r@kadu.net)
  * %kadu copyright end%
  *
@@ -33,38 +34,37 @@
 #include <QtCore/QDir>
 #include <QtCore/QLibraryInfo>
 #include <QtCore/QLocale>
-#include <QtCore/QTimer>
 #include <QtCore/QString>
 #include <QtCore/QStringList>
+#include <QtCore/QTimer>
 #include <QtCore/QTranslator>
 #include <QtGui/QApplication>
 
-#include <time.h>
 #include <errno.h>
+#include <time.h>
 #ifndef Q_WS_WIN
+#include <unistd.h>
 #include <sys/file.h>
 #include <sys/stat.h>
+#include <sys/types.h>
 #include <pwd.h>
 #else // !Q_WS_WIN
 #include <winsock2.h>
 #endif // !Q_WS_WIN
-#if defined(Q_OS_BSD4) || defined(Q_OS_LINUX)
-#include <sys/types.h>
-#endif // Q_OS_BSD4 || Q_OS_LINUX
 
-#include "core/core.h"
 #include "configuration/configuration-file.h"
 #include "configuration/xml-configuration-file.h"
+#include "core/core.h"
 #include "gui/windows/message-dialog.h"
 #include "os/qtsingleapplication/qtlocalpeer.h"
 #include "plugins/plugins-manager.h"
 #include "protocols/protocols-manager.h"
 
-#include "debug.h"
-#include "kadu-config.h"
 #include "icons/icons-manager.h"
-#include "kadu-application.h"
 #include "misc/misc.h"
+#include "debug.h"
+#include "kadu-application.h"
+#include "kadu-config.h"
 
 #ifndef Q_WS_WIN
 static void kaduQtMessageHandler(QtMsgType type, const char *msg)
@@ -204,7 +204,7 @@ int main(int argc, char *argv[])
 	debug_mask = -2;
 
 	kdebugm(KDEBUG_INFO, "before creation of new KaduApplication\n");
-	(void)new KaduApplication(argc, argv);
+	KaduApplication *kaduApplication = new KaduApplication(argc, argv);
 	kdebugm(KDEBUG_INFO, "after creation of new KaduApplication\n");
 
 	for (int i = 1; i < qApp->argc(); ++i)
@@ -356,7 +356,7 @@ int main(int argc, char *argv[])
 	QtLocalPeer *peer = new QtLocalPeer(qApp, profilePath());
 	if (peer->isClient())
 	{
-		if (ids.count())
+		if (!ids.isEmpty())
 			foreach (const QString &id, ids)
 				peer->sendMessage(id, 1000);
 		else
@@ -372,6 +372,7 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
+	Core::instance()->setApplication(kaduApplication);
 	Core::instance()->createGui();
 	QObject::connect(peer, SIGNAL(messageReceived(const QString &)),
 			Core::instance(), SLOT(receivedSignal(const QString &)));
