@@ -1,9 +1,9 @@
 /*
  * %kadu copyright begin%
- * Copyright 2008, 2009 Piotr Galiszewski (piotr.galiszewski@kadu.im)
- * Copyright 2005, 2006, 2007 Marcin Ślusarz (joi@kadu.net)
- * Copyright 2008, 2009, 2010, 2011 Rafał Malinowski (rafal.przemyslaw.malinowski@gmail.com)
+ * Copyright 2008, 2009, 2011 Piotr Galiszewski (piotr.galiszewski@kadu.im)
  * Copyright 2008 Michał Podsiadlik (michal@kadu.net)
+ * Copyright 2008, 2009, 2010, 2011 Rafał Malinowski (rafal.przemyslaw.malinowski@gmail.com)
+ * Copyright 2005, 2006, 2007 Marcin Ślusarz (joi@kadu.net)
  * %kadu copyright end%
  *
  * This program is free software; you can redistribute it and/or
@@ -36,14 +36,16 @@ ProtocolsManager * ProtocolsManager::Instance = 0;
 ProtocolsManager * ProtocolsManager::instance()
 {
 	if (0 == Instance)
+	{
 		Instance = new ProtocolsManager();
+		Instance->init();
+	}
 
 	return Instance;
 }
 
 ProtocolsManager::ProtocolsManager()
 {
-	ConnectionErrorNotification::registerEvent();
 }
 
 ProtocolsManager::~ProtocolsManager()
@@ -51,6 +53,11 @@ ProtocolsManager::~ProtocolsManager()
 	ConnectionErrorNotification::unregisterEvent();
 }
 
+void ProtocolsManager::init()
+{
+	ConnectionErrorNotification::registerEvent();
+}
+
 void ProtocolsManager::registerProtocolFactory(ProtocolFactory *factory)
 {
 	if (!factory || Factories.contains(factory))
