@@ -1,9 +1,10 @@
 /*
  * %kadu copyright begin%
- * Copyright 2011 Bartosz Brachaczek (b.brachaczek@gmail.com)
- * Copyright 2010 Piotr Galiszewski (piotr.galiszewski@kadu.im)
- * Copyright 2009, 2010 Rafał Malinowski (rafal.przemyslaw.malinowski@gmail.com)
+ * Copyright 2010, 2010, 2011 Piotr Galiszewski (piotr.galiszewski@kadu.im)
+ * Copyright 2009 Wojciech Treter (juzefwt@gmail.com)
  * Copyright 2010 Tomasz Rostański (rozteck@interia.pl)
+ * Copyright 2009, 2010 Rafał Malinowski (rafal.przemyslaw.malinowski@gmail.com)
+ * Copyright 2011 Bartosz Brachaczek (b.brachaczek@gmail.com)
  * %kadu copyright end%
  *
  * This program is free software; you can redistribute it and/or
@@ -118,6 +119,93 @@
 	KaduShared_PropertyBoolRead(capitalized_name) \
 	KaduShared_PropertyBoolWrite(capitalized_name)
 
+/**
+ * @author Bartosz 'beevvy' Brachaczek
+ * @short Declares getter for given property of StorableObject.
+ * @param type type of property
+ * @param name name of getter
+ *
+ * Declares getter for given property of @link StorableObject @endlink.
+ */
+#define KaduShared_PropertyReadDecl(type, name) \
+	type name();
+
+/**
+ * @author Bartosz 'beevvy' Brachaczek
+ * @short Declares setter for given property of StorableObject.
+ * @param type type of property
+ * @param name name of value parameter
+ * @param capitalized_name name of property (setter will be called set##capitalized_name)
+ *
+ * Declares setter for given property of @link StorableObject @endlink. Setter is called set##capitalized_name).
+ */
+#define KaduShared_PropertyWriteDecl(type, name, capitalized_name) \
+	void set##capitalized_name(type name);
+
+/**
+ * @author Bartosz 'beevvy' Brachaczek
+ * @short Declares getter and setter for given property of StorableObject.
+ * @param type type of property
+ * @param name name of value parameter
+ * @param capitalized_name name of property.
+ *
+ * Declares both getter and setter for given property of @link StorableObject @endlink.
+ * Getter is called name, setter is called set##capitalized_name).
+ * Argument of the setter will be a const reference to 'type'.
+ */
+#define KaduShared_PropertyDeclCRW(type, name, capitalized_name) \
+	KaduShared_PropertyReadDecl(type, name) \
+	KaduShared_PropertyWriteDecl(const type &, name, capitalized_name)
+
+/**
+ * @author Bartosz 'beevvy' Brachaczek
+ * @short Defines getter for given pointed property of StorableObject.
+ * @param class_name class name
+ * @param type type of property
+ * @param name name of getter
+ * @param capitalized_name name of property
+ *
+ * Defines getter for given pointed property of @link StorableObject @endlink. Value of property is always
+ * valid because this method calls @link<StorableObject::ensureLoaded @endLink, so object is always
+ * fully loaded before this method returns. Getter is called name.
+ */
+#define KaduShared_PropertyPtrReadDef(class_name, type, name, capitalized_name) \
+	type class_name::name() { ensureLoaded(); return *capitalized_name; }
+
+/**
+ * @author Bartosz 'beevvy' Brachaczek
+ * @short Defines setter for given pointed property of StorableObject.
+ * @param class_name class name
+ * @param type type of property
+ * @param name name of value parameter
+ * @param capitalized_name name of property (setter will be called set##capitalized_name)
+ *
+ * Defines setter for given pointed property of @link StorableObject @endlink. Value of property will not be
+ * ovveriden by load, because this method calls @link<StorableObject::ensureLoaded @endLink, so object is always
+ * fully loaded before this method returns. Setter is called set##capitalized_name).
+ * Calls @link dataUpdated @endlink method after data is changed.
+ */
+#define KaduShared_PropertyPtrWriteDef(class_name, type, name, capitalized_name) \
+	void class_name::set##capitalized_name(type name) { ensureLoaded(); if (*capitalized_name != name) { *capitalized_name = name; dataUpdated(); } }
+
+/**
+ * @author Bartosz 'beevvy' Brachaczek
+ * @short Defines getter and setter for given pointed property of StorableObject.
+ * @param class_name class name
+ * @param type type of property
+ * @param name name of value parameter
+ * @param capitalized_name name of property.
+ *
+ * Defines both getter and setter for given pointed property of @link StorableObject @endlink. Value of property will
+ * always be valid, because these methods calls @link<StorableObject::ensureLoaded @endLink, so object is always
+ * fully loaded before this method returns. Getter is called name, setter is called set##capitalized_name).
+ * Setter calls @link dataUpdated @endlink method after data is changed.
+ * Argument of the setter will be a const reference to 'type'.
+ */
+#define KaduShared_PropertyPtrDefCRW(class_name, type, name, capitalized_name) \
+	KaduShared_PropertyPtrReadDef(class_name, type, name, capitalized_name) \
+	KaduShared_PropertyPtrWriteDef(class_name, const type &, name, capitalized_name)
+
 /**
  * @author Rafal 'Vogel' Malinowski
  * @class Shared
@@ -152,6 +240,7 @@ class KADUAPI Shared : public UuidStorableObject, public QSharedData
 
 protected:
 	virtual void load();
+	virtual void store();
 	void loadStub();
 
 	void dataUpdated();
@@ -161,7 +250,6 @@ public:
 	explicit Shared(const QUuid &uuid);
 	virtual ~Shared();
 
-	virtual void store();
 	virtual void aboutToBeRemoved();
 
 	void blockUpdatedSignal();
