@@ -1,10 +1,10 @@
 /*
  * %kadu copyright begin%
- * Copyright 2010, 2011 Bartosz Brachaczek (b.brachaczek@gmail.com)
- * Copyright 2009 Wojciech Treter (juzefwt@gmail.com)
  * Copyright 2009, 2010, 2011 Piotr Galiszewski (piotr.galiszewski@kadu.im)
- * Copyright 2009, 2010, 2011 Rafał Malinowski (rafal.przemyslaw.malinowski@gmail.com)
+ * Copyright 2009 Wojciech Treter (juzefwt@gmail.com)
  * Copyright 2009 Michał Podsiadlik (michal@kadu.net)
+ * Copyright 2009, 2010, 2011 Rafał Malinowski (rafal.przemyslaw.malinowski@gmail.com)
+ * Copyright 2010, 2011 Bartosz Brachaczek (b.brachaczek@gmail.com)
  * %kadu copyright end%
  *
  * This program is free software; you can redistribute it and/or
@@ -23,15 +23,17 @@
 
 #include <QtCore/QFile>
 
-#include "debug.h"
 #include "misc/misc.h"
+#include "debug.h"
 
 #include "file-transfer/gadu-file-transfer-handler.h"
 #include "socket-notifiers/gadu-protocol-socket-notifiers.h"
 
 #include "dcc-socket-notifiers.h"
 
-#ifdef Q_OS_WIN
+#ifndef Q_OS_WIN
+#include <unistd.h>
+#else
 #define dup(x) x
 #endif
 
