@@ -1,6 +1,17 @@
 <?xml version="1.0" encoding="utf-8"?>
 <!DOCTYPE TS>
 <TS version="2.0" language="de">
+<context>
+    <name>GrowlNotify</name>
+    <message>
+        <source>Error</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Growl is not installed in your system</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
 <context>
     <name>GrowlNotifyConfigurationWidget</name>
     <message>
