@@ -1,11 +1,11 @@
 /*
  * %kadu copyright begin%
- * Copyright 2010, 2011 Bartosz Brachaczek (b.brachaczek@gmail.com)
- * Copyright 2009, 2010, 2011 Piotr Galiszewski (piotr.galiszewski@kadu.im)
- * Copyright 2009, 2010 Wojciech Treter (juzefwt@gmail.com)
- * Copyright 2009, 2010, 2011 Rafał Malinowski (rafal.przemyslaw.malinowski@gmail.com)
+ * Copyright 2009, 2010, 2010, 2011 Piotr Galiszewski (piotr.galiszewski@kadu.im)
+ * Copyright 2009, 2009, 2009, 2009, 2010 Wojciech Treter (juzefwt@gmail.com)
  * Copyright 2009 Michał Podsiadlik (michal@kadu.net)
  * Copyright 2009 Bartłomiej Zimoń (uzi18@o2.pl)
+ * Copyright 2009, 2010, 2011 Rafał Malinowski (rafal.przemyslaw.malinowski@gmail.com)
+ * Copyright 2010, 2011 Bartosz Brachaczek (b.brachaczek@gmail.com)
  * %kadu copyright end%
  *
  * This program is free software; you can redistribute it and/or
@@ -24,213 +24,156 @@
 
 #include <QtCore/QDir>
 #include <QtCore/QMutexLocker>
+#include <QtCore/QThread>
+#include <QtCore/QtConcurrentRun>
 #include <QtGui/QTextDocument>
 #include <QtSql/QSqlError>
 #include <QtSql/QSqlRecord>
 
-#include "accounts/account.h"
 #include "accounts/account-manager.h"
+#include "accounts/account.h"
 #include "buddies/buddy-manager.h"
-#include "buddies/buddy-shared.h"
-#include "chat/chat-details.h"
 #include "chat/chat-details-aggregate.h"
+#include "chat/chat-details.h"
 #include "chat/chat-manager.h"
-#include "chat/message/formatted-message.h"
-#include "chat/message/message.h"
 #include "configuration/configuration-file.h"
 #include "contacts/contact-manager.h"
 #include "contacts/contact-set.h"
 #include "core/core.h"
-#include "debug.h"
+#include "gui/widgets/chat-widget.h"
 #include "gui/windows/message-dialog.h"
+#include "gui/windows/progress-window2.h"
+#include "message/formatted-message.h"
+#include "message/message.h"
 #include "misc/misc.h"
 #include "misc/path-conversion.h"
-#include "gui/widgets/chat-widget.h"
+#include "status/status-type-manager.h"
+#include "debug.h"
 
-#include "plugins/history/model/dates-model-item.h"
 #include "plugins/history/history.h"
+#include "plugins/history/model/dates-model-item.h"
 #include "plugins/history/search/history-search-parameters.h"
 #include "plugins/history/timed-status.h"
 
+#include "storage/sql-initializer.h"
+
 #include "history-sql-storage.h"
 
+#define DATE_TITLE_LENGTH 120
+
 HistorySqlStorage::HistorySqlStorage(QObject *parent) :
-		HistoryStorage(parent), DatabaseMutex(QMutex::NonRecursive)
-{
-	kdebugf();
-
-	QMutexLocker locker(&DatabaseMutex);
-
-	initDatabase();
-	initQueries();
-}
-
-HistorySqlStorage::~HistorySqlStorage()
-{
-	kdebugf();
-
-	Database.commit();
-}
-
-void HistorySqlStorage::initDatabase()
+		HistoryStorage(parent), ImportProgressWindow(0), DatabaseMutex(QMutex::NonRecursive)
 {
 	kdebugf();
 
 	if (!QSqlDatabase::isDriverAvailable("QSQLITE"))
 	{
 		MessageDialog::show(KaduIcon("dialog-warning"), tr("Kadu"),
-				tr("It seems your Qt library does not provide support for selected database.\n "
-				   "Please select another driver in configuration window or install Qt with %1 plugin.").arg("QSQLITE"));
+				tr("It seems your Qt library does not provide support for selected database. "
+				   "Please install Qt with %1 plugin.").arg("QSQLITE"));
+		return;
+	}
+
+	qRegisterMetaType<QSqlError>("QSqlError");
+
+	InitializerThread = new QThread();
+
+	// this object cannot have parent as it will be moved to a new thread
+	SqlInitializer *initializer = new SqlInitializer();
+	initializer->moveToThread(InitializerThread);
+
+	connect(InitializerThread, SIGNAL(started()), initializer, SLOT(initialize()));
+	connect(initializer, SIGNAL(initialized()), InitializerThread, SLOT(quit()));
+	connect(initializer, SIGNAL(databaseReady(bool)), this, SLOT(databaseReady(bool)));
+	connect(initializer, SIGNAL(importStarted()), this, SLOT(importStarted()));
+	connect(initializer, SIGNAL(importFinished()), this, SLOT(importFinished()));
+	connect(initializer, SIGNAL(databaseOpenFailed(QSqlError)), this, SLOT(databaseOpenFailed(QSqlError)));
+
+	InitializerThread->start();
+
+	History::instance()->registerStorage(this);
+}
+
+HistorySqlStorage::~HistorySqlStorage()
+{
+	kdebugf();
+
+	if (InitializerThread && InitializerThread->isRunning())
+	{
+		InitializerThread->terminate();
+		InitializerThread->wait(2000);
+	}
+
+	delete InitializerThread;
+	InitializerThread = 0;
+
+	if (Database.isOpen())
+		Database.commit();
+}
+
+void HistorySqlStorage::databaseReady(bool ok)
+{
+	if (ok)
+		Database = QSqlDatabase::database("kadu-history", true);
+
+	if (!Database.isOpen())
+	{
+		databaseOpenFailed(Database.lastError());
 		History::instance()->unregisterStorage(this);
 		return;
 	}
 
-	if (QSqlDatabase::contains("kadu-history"))
-	{
-		if (Database.isOpen())
-			Database.close();
-		QSqlDatabase::removeDatabase("kadu-history");
-	}
-
-	QDir historyDir(profilePath("history"));
-	if (!historyDir.exists())
-		historyDir.mkpath(profilePath("history"));
-
-	Database = QSqlDatabase::addDatabase("QSQLITE", "kadu-history");
-	Database.setDatabaseName(profilePath("history/history.db"));
-
-	if (!Database.open())
-	{
-		MessageDialog::show(KaduIcon("dialog-warning"), tr("Kadu"), Database.lastError().text());
-		return;
-	}
-
 	Database.transaction();
-
-	initTables();
-	initIndexes();
+	initQueries();
 }
 
-void HistorySqlStorage::initTables()
+void HistorySqlStorage::importStarted()
 {
-	if (!Database.tables().contains("kadu_messages"))
-		initKaduMessagesTable();
-	if (!Database.tables().contains("kadu_statuses"))
-		initKaduStatusesTable();
-	if (!Database.tables().contains("kadu_sms"))
-		initKaduSmsTable();
-}
-
-void HistorySqlStorage::initKaduMessagesTable()
-{
-	QSqlQuery query(Database);
-
-	query.prepare("PRAGMA encoding = \"UTF-8\";");
-	executeQuery(query);
-
-	query.prepare("PRAGMA synchronous = OFF;");
-	executeQuery(query);
-
-	query.prepare(
-		"CREATE TABLE kadu_messages ("
-			"chat VARCHAR(255),"
-			"sender VARCHAR(255),"
-			"send_time TIMESTAMP,"
-			"receive_time TIMESTAMP,"
-			"content TEXT,"
-			"attributes TEXT);"
+	ImportProgressWindow = new ProgressWindow2(
+	            tr("Optimizing history database. This can take several minutes.\n"
+	               "Please do not close Kadu until optimalization is complete.")
 	);
-	executeQuery(query);
+	ImportProgressWindow->show();
 }
 
-void HistorySqlStorage::initKaduStatusesTable()
+void HistorySqlStorage::importFinished()
 {
-	QSqlQuery query(Database);
-
-	query.prepare("PRAGMA encoding = \"UTF-8\";");
-	executeQuery(query);
-
-	query.prepare("PRAGMA synchronous = OFF;");
-	executeQuery(query);
-
-	query.prepare(
-		"CREATE TABLE kadu_statuses ("
-			"contact VARCHAR(255),"
-			"status VARCHAR(255),"
-			"set_time TIMESTAMP,"
-			"description TEXT);"
-	);
-	executeQuery(query);
+	if (ImportProgressWindow)
+	{
+		ImportProgressWindow->setText(tr("Optimalization complete. You can now close this window."));
+		ImportProgressWindow->enableClosing();
+	}
 }
 
-void HistorySqlStorage::initKaduSmsTable()
+void HistorySqlStorage::databaseOpenFailed (const QSqlError &error)
 {
-	QSqlQuery query(Database);
-
-	query.prepare("PRAGMA encoding = \"UTF-8\";");
-	executeQuery(query);
-
-	query.prepare("PRAGMA synchronous = OFF;");
-	executeQuery(query);
-
-	query.prepare(
-		"CREATE TABLE kadu_sms ("
-			"receipient VARCHAR(255),"
-			"send_time TIMESTAMP,"
-			"content TEXT);"
-	);
-	executeQuery(query);
+	if (ImportProgressWindow)
+	{
+		ImportProgressWindow->setText(tr("Optimalization failed. Error message:\n%1").arg(error.text()));
+		ImportProgressWindow->enableClosing();
+	}
+	else
+		MessageDialog::show(KaduIcon("dialog-warning"), tr("Kadu"), error.text());
 }
 
-void HistorySqlStorage::initIndexes()
+bool HistorySqlStorage::isDatabaseReady(bool wait)
 {
-	QSqlQuery query(Database);
+	if (InitializerThread && InitializerThread->isRunning())
+	{
+		if (wait)
+			InitializerThread->wait();
+		else
+			return false;
+	}
 
-	query.prepare("CREATE INDEX IF NOT EXISTS kadu_messages_chat ON kadu_messages (chat)");
-	executeQuery(query);
-
-	query.prepare("CREATE INDEX IF NOT EXISTS kadu_messages_chat_receive_time_rowid ON kadu_messages (chat, receive_time, rowid)");
-	executeQuery(query);
-
-	query.prepare("DROP INDEX IF EXISTS kadu_messages_chat_receive_time");
-	executeQuery(query);
-
-	query.prepare("DROP INDEX IF EXISTS kadu_messages_chat_receive_time_date");
-	executeQuery(query);
-
-	query.prepare("DROP INDEX IF EXISTS kadu_messages_chat_receive_time_send_time");
-	executeQuery(query);
-
-	query.prepare("DROP INDEX IF EXISTS kadu_messages_chat_receive_time_date_send_time");
-	executeQuery(query);
-
-	query.prepare("CREATE INDEX IF NOT EXISTS kadu_statuses_contact ON kadu_statuses (contact)");
-	executeQuery(query);
-
-	query.prepare("CREATE INDEX IF NOT EXISTS kadu_statuses_contact_time ON kadu_statuses (contact, set_time)");
-	executeQuery(query);
-
-	query.prepare("DROP INDEX IF EXISTS kadu_statuses_contact_time_date");
-	executeQuery(query);
-
-	query.prepare("CREATE INDEX IF NOT EXISTS kadu_sms_receipient ON kadu_sms (receipient)");
-	executeQuery(query);
-
-	query.prepare("CREATE INDEX IF NOT EXISTS kadu_sms_receipient_time ON kadu_sms (receipient, send_time)");
-	executeQuery(query);
-
-	query.prepare("DROP INDEX IF EXISTS kadu_sms_receipient_time_date");
-	executeQuery(query);
+	return Database.isOpen();
 }
 
 void HistorySqlStorage::initQueries()
 {
-	ListChatsQuery = QSqlQuery(Database);
-	ListChatsQuery.prepare("SELECT DISTINCT chat FROM kadu_messages");
-
 	AppendMessageQuery = QSqlQuery(Database);
-	AppendMessageQuery.prepare("INSERT INTO kadu_messages (chat, sender, send_time, receive_time, content, attributes) VALUES "
-			"(:chat, :sender, :send_time, :receive_time, :content, :attributes)");
+	AppendMessageQuery.prepare("INSERT INTO kadu_messages (chat_id, contact_id, send_time, receive_time, date_id, is_outgoing, content_id) VALUES "
+			"(:chat_id, :contact_id, :send_time, :receive_time, :date_id, :is_outgoing, :content_id)");
 
 	AppendStatusQuery = QSqlQuery(Database);
 	AppendStatusQuery.prepare("INSERT INTO kadu_statuses (contact, status, set_time, description) VALUES "
@@ -241,7 +184,7 @@ void HistorySqlStorage::initQueries()
 			"(:receipient, :send_time, :content)");
 }
 
-QString HistorySqlStorage::chatWhere(const Chat &chat)
+QString HistorySqlStorage::chatWhere(const Chat &chat, const QString &chatPrefix)
 {
 	if (!chat)
 		return QLatin1String("false");
@@ -252,16 +195,16 @@ QString HistorySqlStorage::chatWhere(const Chat &chat)
 
 	ChatDetailsAggregate *aggregate = qobject_cast<ChatDetailsAggregate *>(details);
 	if (!aggregate)
-		return QString("chat = '%1'").arg(chat.uuid().toString());
+		return QString("%1uuid = '%2'").arg(chatPrefix).arg(chat.uuid().toString());
 
 	QStringList uuids;
 	foreach (const Chat &aggregatedChat, aggregate->chats())
 		uuids.append(QString("'%1'").arg(aggregatedChat.uuid().toString()));
 
-	return QString("chat IN (%1)").arg(uuids.join(QLatin1String(", ")));
+	return QString("%1uuid IN (%2)").arg(chatPrefix).arg(uuids.join(QLatin1String(", ")));
 }
 
-QString HistorySqlStorage::buddyContactsWhere(const Buddy &buddy)
+QString HistorySqlStorage::buddyContactsWhere(const Buddy &buddy, const QString &fieldName)
 {
 	if (!buddy || buddy.contacts().isEmpty())
 		return  QLatin1String("false");
@@ -270,11 +213,14 @@ QString HistorySqlStorage::buddyContactsWhere(const Buddy &buddy)
 	foreach (const Contact &contact, buddy.contacts())
 		uuids.append(QString("'%1'").arg(contact.uuid().toString()));
 
-	return QString("contact IN (%1)").arg(uuids.join(QLatin1String(", ")));
+	return QString("(%1) IN (%2)").arg(fieldName).arg(uuids.join(QLatin1String(", ")));
 }
 
 void HistorySqlStorage::sync()
 {
+	if (!isDatabaseReady(false))
+		return; // nothing to sync yet
+
 	QMutexLocker locker(&DatabaseMutex);
 
 	Database.commit();
@@ -291,22 +237,153 @@ void HistorySqlStorage::messageSent(const Message &message)
 	appendMessage(message);
 }
 
+int HistorySqlStorage::findOrCreateChat(const Chat &chat)
+{
+	if (ChatMap.contains(chat))
+		return ChatMap.value(chat);
+
+	QSqlQuery query(Database);
+	QString queryString = "SELECT id FROM kadu_chats WHERE uuid=:uuid";
+
+	query.prepare(queryString);
+	query.bindValue(":uuid", chat.uuid().toString());
+
+	int chatId = -1;
+
+	executeQuery(query);
+
+	if (query.next())
+	{
+		chatId = query.value(0).toInt();
+		Q_ASSERT(!query.next());
+	}
+	else
+	{
+		QSqlQuery query(Database);
+		QString queryString = "INSERT INTO kadu_chats (uuid) VALUES (:uuid)";
+
+		query.prepare(queryString);
+		query.bindValue(":uuid", chat.uuid().toString());
+
+		executeQuery(query);
+		chatId = query.lastInsertId().toInt();
+	}
+
+	ChatMap.insert(chat, chatId);
+
+	return chatId;
+}
+
+int HistorySqlStorage::findOrCreateContact(const Contact &contact)
+{
+	if (ContactMap.contains(contact))
+		return ContactMap.value(contact);
+
+	QSqlQuery query(Database);
+	QString queryString = "SELECT id FROM kadu_contacts WHERE uuid=:uuid";
+
+	query.prepare(queryString);
+	query.bindValue(":uuid", contact.uuid().toString());
+
+	int contactId = -1;
+
+	executeQuery(query);
+
+	if (query.next())
+	{
+		contactId = query.value(0).toInt();
+		Q_ASSERT(!query.next());
+	}
+	else
+	{
+		QSqlQuery query(Database);
+		QString queryString = "INSERT INTO kadu_contacts (uuid) VALUES (:uuid)";
+
+		query.prepare(queryString);
+		query.bindValue(":uuid", contact.uuid().toString());
+
+		executeQuery(query);
+		contactId = query.lastInsertId().toInt();
+	}
+
+	ContactMap.insert(contact, contactId);
+
+	return contactId;
+}
+
+int HistorySqlStorage::findOrCreateDate(const QDate &date)
+{
+	QString stringDate = date.toString("yyyyMMdd");
+	if (DateMap.contains(stringDate))
+		return DateMap.value(stringDate);
+
+	QSqlQuery query(Database);
+	QString queryString = "SELECT id FROM kadu_dates WHERE date=:date";
+
+	query.prepare(queryString);
+	query.bindValue(":date", stringDate);
+
+	int dateId = -1;
+
+	executeQuery(query);
+
+	if (query.next())
+	{
+		dateId = query.value(0).toInt();
+		Q_ASSERT(!query.next());
+	}
+	else
+	{
+		QSqlQuery query(Database);
+		QString queryString = "INSERT INTO kadu_dates (date) VALUES (:date)";
+
+		query.prepare(queryString);
+		query.bindValue(":date", stringDate);
+
+		executeQuery(query);
+		dateId = query.lastInsertId().toInt();
+	}
+
+	DateMap.insert(stringDate, dateId);
+
+	return dateId;
+}
+
+int HistorySqlStorage::saveMessageContent(const Message& message)
+{
+	QSqlQuery saveMessageQuery = QSqlQuery(Database);
+	saveMessageQuery.prepare("INSERT INTO kadu_message_contents (content) VALUES (:content)");
+
+	saveMessageQuery.bindValue(":content", message.content());
+
+	executeQuery(saveMessageQuery);
+	int contentId = saveMessageQuery.lastInsertId().toInt();
+
+	saveMessageQuery.finish();
+
+	return contentId;
+}
+
 void HistorySqlStorage::appendMessage(const Message &message)
 {
 	kdebugf();
 
+	if (!isDatabaseReady(true))
+		return;
+
 	QMutexLocker locker(&DatabaseMutex);
 
-	QString outgoing = (message.type() == MessageTypeSent)
-			? "1"
-			: "0";
+	int outgoing = (message.type() == MessageTypeSent)
+			? 1
+			: 0;
 
-	AppendMessageQuery.bindValue(":chat", message.messageChat().uuid().toString());
-	AppendMessageQuery.bindValue(":sender", message.messageSender().uuid().toString());
+	AppendMessageQuery.bindValue(":chat_id", findOrCreateChat(message.messageChat()));
+	AppendMessageQuery.bindValue(":contact_id", findOrCreateContact(message.messageSender()));
 	AppendMessageQuery.bindValue(":send_time", message.sendDate());
 	AppendMessageQuery.bindValue(":receive_time", message.receiveDate());
-	AppendMessageQuery.bindValue(":content", message.content());
-	AppendMessageQuery.bindValue(":attributes", QString("outgoing=%1").arg(outgoing));
+	AppendMessageQuery.bindValue(":date_id", findOrCreateDate(message.receiveDate().date()));
+	AppendMessageQuery.bindValue(":is_outgoing", outgoing);
+	AppendMessageQuery.bindValue(":content_id", saveMessageContent(message));
 
 	executeQuery(AppendMessageQuery);
 
@@ -319,6 +396,9 @@ void HistorySqlStorage::appendStatus(const Contact &contact, const Status &statu
 {
 	kdebugf();
 
+	if (!isDatabaseReady(true))
+		return;
+
 	QMutexLocker locker(&DatabaseMutex);
 
 	AppendStatusQuery.bindValue(":contact", contact.uuid().toString());
@@ -337,6 +417,9 @@ void HistorySqlStorage::appendSms(const QString &recipient, const QString &conte
 {
 	kdebugf();
 
+	if (!isDatabaseReady(true))
+		return;
+
 	QMutexLocker locker(&DatabaseMutex);
 
 	AppendSmsQuery.bindValue(":contact", recipient);
@@ -352,27 +435,42 @@ void HistorySqlStorage::appendSms(const QString &recipient, const QString &conte
 
 void HistorySqlStorage::clearChatHistory(const Chat &chat, const QDate &date)
 {
+	if (!isDatabaseReady(true))
+		return;
+
 	QMutexLocker locker(&DatabaseMutex);
 
 	QSqlQuery query(Database);
-	QString queryString = "DELETE FROM kadu_messages WHERE " + chatWhere(chat);
+	QString queryString = "DELETE FROM kadu_messages WHERE chat_id IN (SELECT id FROM kadu_chats chat WHERE " + chatWhere(chat) + ")";
 	if (!date.isNull())
-		queryString += " AND substr(receive_time,0,11) = :date";
+		queryString += " AND date_id IN (SELECT id FROM kadu_dates WHERE date = :date)";
 
 	query.prepare(queryString);
 
 	if (!date.isNull())
-		query.bindValue(":date", date.toString(Qt::ISODate));
+		query.bindValue(":date", date.toString("yyyyMMdd"));
 
 	executeQuery(query);
+
+	QString removeChatsQueryString = "DELETE FROM kadu_chats WHERE " + chatWhere(chat, "") +
+	        " AND 0 = (SELECT count(*) FROM kadu_messages WHERE chat_id = kadu_chats.id)";
+
+	QSqlQuery removeChatsQuery(Database);
+
+	removeChatsQuery.prepare(removeChatsQueryString);
+
+	executeQuery(removeChatsQuery);
 }
 
 void HistorySqlStorage::clearStatusHistory(const Buddy &buddy, const QDate &date)
 {
+	if (!isDatabaseReady(true))
+		return;
+
 	QMutexLocker locker(&DatabaseMutex);
 
 	QSqlQuery query(Database);
-	QString queryString = "DELETE FROM kadu_statuses WHERE " + buddyContactsWhere(buddy);
+	QString queryString = "DELETE FROM kadu_statuses WHERE " + buddyContactsWhere(buddy, "contact");
 	if (!date.isNull())
 		queryString += " AND substr(set_time,0,11) = :date";
 
@@ -386,6 +484,9 @@ void HistorySqlStorage::clearStatusHistory(const Buddy &buddy, const QDate &date
 
 void HistorySqlStorage::clearSmsHistory(const QString &recipient, const QDate &date)
 {
+	if (!isDatabaseReady(true))
+		return;
+
 	QMutexLocker locker(&DatabaseMutex);
 
 	QSqlQuery query(Database);
@@ -404,6 +505,9 @@ void HistorySqlStorage::clearSmsHistory(const QString &recipient, const QDate &d
 
 void HistorySqlStorage::deleteHistory(const Buddy &buddy)
 {
+	if (!isDatabaseReady(true))
+		return;
+
 	QMutexLocker locker(&DatabaseMutex);
 
 	QSqlQuery query(Database);
@@ -413,43 +517,50 @@ void HistorySqlStorage::deleteHistory(const Buddy &buddy)
 		Chat chat = ChatManager::instance()->findChat(ContactSet(contact), false);
 		if (chat)
 		{
-			QString queryString = "DELETE FROM kadu_messages WHERE " + chatWhere(chat);
+			QString queryString = "DELETE FROM kadu_messages LEFT JOIN kadu_chats chat ON (kadu_messages.chat_id=chat.id) WHERE " + chatWhere(chat);
 			query.prepare(queryString);
 			executeQuery(query);
 		}
 	}
 
-	QString queryString = "DELETE FROM kadu_statuses WHERE " + buddyContactsWhere(buddy);
+	QString queryString = "DELETE FROM kadu_statuses WHERE " + buddyContactsWhere(buddy, "contact");
 	query.prepare(queryString);
 	executeQuery(query);
 }
 
-QList<Chat> HistorySqlStorage::chats(const HistorySearchParameters &search)
+QVector<Chat> HistorySqlStorage::chats(const HistorySearchParameters &search)
 {
 	kdebugf();
 
+	if (!isDatabaseReady(false))
+		return QVector<Chat>();
+
 	QMutexLocker locker(&DatabaseMutex);
 
+	QString joins = !search.query().isEmpty() || search.fromDate().isValid() || search.toDate().isValid()
+		? "LEFT JOIN kadu_messages km ON (kadu_chats.id=km.chat_id) LEFT JOIN kadu_dates kd ON (kd.id=km.date_id) LEFT JOIN kadu_message_contents kmc ON (kmc.id=km.content_id) "
+		: "";
+
 	QSqlQuery query(Database);
-	QString queryString = "SELECT DISTINCT chat FROM kadu_messages WHERE 1";
+	QString queryString = "SELECT uuid FROM kadu_chats " + joins + "WHERE 1";
 
 	if (!search.query().isEmpty())
 		queryString += " AND content LIKE :content";
 	if (search.fromDate().isValid())
-		queryString += " AND substr(receive_time,0,11) >= :fromDate";
+		queryString += " AND date >= :fromDate";
 	if (search.toDate().isValid())
-		queryString += " AND substr(receive_time,0,11) <= :toDate";
+		queryString += " AND date <= :toDate";
 
 	query.prepare(queryString);
 
 	if (!search.query().isEmpty())
 		query.bindValue(":content", QString('%' + search.query() + '%'));
 	if (search.fromDate().isValid())
-		query.bindValue(":fromDate", search.fromDate());
+		query.bindValue(":fromDate", search.fromDate().toString("yyyyMMdd"));
 	if (search.toDate().isValid())
-		query.bindValue(":toDate", search.toDate());
+		query.bindValue(":toDate", search.toDate().toString("yyyyMMdd"));
 
-	QList<Chat> chats;
+	QVector<Chat> chats;
 
 	executeQuery(query);
 
@@ -463,38 +574,44 @@ QList<Chat> HistorySqlStorage::chats(const HistorySearchParameters &search)
 	return chats;
 }
 
-QList<DatesModelItem> HistorySqlStorage::chatDates(const Chat &chat, const HistorySearchParameters &search)
+QVector<DatesModelItem> HistorySqlStorage::chatDates(const Chat &chat, const HistorySearchParameters &search)
 {
 	kdebugf();
 
 	if (!chat)
-		return QList<DatesModelItem>();
+		return QVector<DatesModelItem>();
+
+	if (!isDatabaseReady(false))
+		return QVector<DatesModelItem>();
 
 	QMutexLocker locker(&DatabaseMutex);
 
 	QSqlQuery query(Database);
-	QString queryString = "SELECT count(1), receive_time, content FROM";
-	queryString += " (SELECT receive_time, content from kadu_messages WHERE " + chatWhere(chat);
+	QString queryString = "SELECT count(1), date, content FROM";
+	queryString += " (SELECT km.rowid, date, date_id, content FROM kadu_messages km "
+		"LEFT JOIN kadu_message_contents kmc ON (km.content_id=kmc.id) "
+		"LEFT JOIN kadu_dates d ON (km.date_id=d.id) "
+		"LEFT JOIN kadu_chats chat ON (km.chat_id=chat.id) WHERE " + chatWhere(chat);
 	if (!search.query().isEmpty())
-		queryString += " AND content LIKE :content";
+	  queryString += " AND kmc.content LIKE :content";
 	if (search.fromDate().isValid())
-		queryString += " AND substr(receive_time,0,11) >= :fromDate";
+	  queryString += " AND date >= :fromDate";
 	if (search.toDate().isValid())
-		queryString += " AND substr(receive_time,0,11) <= :toDate";
-	queryString += " ORDER BY receive_time DESC, rowid DESC )";
-	queryString += " GROUP BY substr(receive_time,0,11)";
-	queryString += " ORDER BY receive_time ASC, rowid ASC";
+	  queryString += " AND date <= :toDate";
+	queryString += " ORDER BY date_id DESC, km.rowid DESC )";
+	queryString += " GROUP BY date_id";
+	queryString += " ORDER BY date_id ASC, rowid ASC";
 
 	query.prepare(queryString);
 
 	if (!search.query().isEmpty())
 		query.bindValue(":content", QString('%' + search.query() + '%'));
 	if (search.fromDate().isValid())
-		query.bindValue(":fromDate", search.fromDate());
+		query.bindValue(":fromDate", search.fromDate().toString("yyyyMMdd"));
 	if (search.toDate().isValid())
-		query.bindValue(":toDate", search.toDate());
+		query.bindValue(":toDate", search.toDate().toString("yyyyMMdd"));
 
-	QList<DatesModelItem> dates;
+	QVector<DatesModelItem> dates;
 	executeQuery(query);
 
 	int count;
@@ -503,7 +620,9 @@ QList<DatesModelItem> HistorySqlStorage::chatDates(const Chat &chat, const Histo
 	while (query.next())
 	{
 		count = query.value(0).toInt();
-		date = query.value(1).toDate();
+
+		QString dateString = query.value(1).toString();
+		date = QDate::fromString(dateString, "yyyyMMdd");
 		if (!date.isValid())
 			continue;
 
@@ -517,37 +636,44 @@ QList<DatesModelItem> HistorySqlStorage::chatDates(const Chat &chat, const Histo
 		FormattedMessage formatted = FormattedMessage::parse(&document);
 		QString title = formatted.toPlain();
 
-		if (title.length() > 20)
+		if (title.length() > DATE_TITLE_LENGTH)
 		{
-			title.truncate(20);
+			title.truncate(DATE_TITLE_LENGTH);
 			title += " ...";
 		}
 
-		dates.append(DatesModelItem(date, count, title));
+		dates.append(DatesModelItem(date, title, count));
 	}
 
 	return dates;
 }
 
-QList<Message> HistorySqlStorage::messages(const Chat &chat, const QDate &date, int limit)
+QVector<Message> HistorySqlStorage::messages(const Chat &chat, const QDate &date, int limit)
 {
 	kdebugf();
 
+	if (!isDatabaseReady(false))
+		return QVector<Message>();
+
 	QMutexLocker locker(&DatabaseMutex);
 
 	QSqlQuery query(Database);
-	QString queryString = "SELECT chat, sender, content, send_time, receive_time, attributes FROM kadu_messages WHERE " + chatWhere(chat);
+	QString queryString = "SELECT chat.uuid, con.uuid, kmc.content, send_time, receive_time, is_outgoing FROM kadu_messages "
+			"LEFT JOIN kadu_chats chat ON (kadu_messages.chat_id=chat.id) "
+			"LEFT JOIN kadu_dates d ON (kadu_messages.date_id=d.id) "
+			"LEFT JOIN kadu_contacts con ON (kadu_messages.contact_id=con.id) "
+			"LEFT JOIN kadu_message_contents kmc ON (kadu_messages.content_id=kmc.id) WHERE " + chatWhere(chat);
 	if (!date.isNull())
-		queryString += " AND substr(receive_time,0,11) = :date";
-	queryString += " ORDER BY receive_time ASC, rowid ASC";
+		queryString += " AND date = :date";
+	queryString += " ORDER BY kadu_messages.date_id ASC, kadu_messages.rowid ASC";
 	if (0 != limit)
 		queryString += " LIMIT :limit";
 
-	QList<Message> messages;
+	QVector<Message> messages;
 	query.prepare(queryString);
 
 	if (!date.isNull())
-		query.bindValue(":date", date.toString(Qt::ISODate));
+		query.bindValue(":date", date.toString("yyyyMMdd"));
 	if (limit != 0)
 		query.bindValue(":limit", limit);
 	executeQuery(query);
@@ -556,23 +682,24 @@ QList<Message> HistorySqlStorage::messages(const Chat &chat, const QDate &date,
 	return messages;
 }
 
-QList<Message> HistorySqlStorage::messagesSince(const Chat &chat, const QDate &date)
+QVector<Message> HistorySqlStorage::getMessagesSince(const Chat &chat, const QDate &date)
 {
-	kdebugf();
-
 	QMutexLocker locker(&DatabaseMutex);
 
-	QList<Message> messages;
+	QVector<Message> messages;
 	if (date.isNull())
 		return messages;
 
 	QSqlQuery query(Database);
-	QString queryString = "SELECT chat, sender, content, send_time, receive_time, attributes FROM kadu_messages WHERE " + chatWhere(chat) +
-			" AND substr(receive_time,0,11) >= :date ORDER BY receive_time ASC, rowid ASC";
+	QString queryString = "SELECT chat.uuid, con.uuid, kmc.content, send_time, receive_time, is_outgoing FROM kadu_messages "
+			"LEFT JOIN kadu_chats chat ON (kadu_messages.chat_id=chat.id) "
+			"LEFT JOIN kadu_dates d ON (kadu_messages.date_id=d.id) "
+			"LEFT JOIN kadu_contacts con ON (kadu_messages.contact_id=con.id) "
+			"LEFT JOIN kadu_message_contents kmc ON (kadu_messages.content_id=kmc.id) WHERE " + chatWhere(chat) +
+			" AND date >= :date ORDER BY date_id ASC, kadu_messages.rowid ASC";
 	query.prepare(queryString);
 
-	query.bindValue(":chat", chat.uuid().toString());
-	query.bindValue(":date", date.toString(Qt::ISODate));
+	query.bindValue(":date", date.toString("yyyyMMdd"));
 
 	executeQuery(query);
 
@@ -581,21 +708,36 @@ QList<Message> HistorySqlStorage::messagesSince(const Chat &chat, const QDate &d
 	return messages;
 }
 
-QList<Message> HistorySqlStorage::messagesBackTo(const Chat &chat, const QDateTime &datetime, int limit)
+QVector<Message> HistorySqlStorage::syncMessagesSince(const Chat &chat, const QDate &date)
+{
+	if (!isDatabaseReady(true))
+		return QVector<Message>();
+
+	return getMessagesSince(chat, date);
+}
+
+QFuture<QVector<Message> > HistorySqlStorage::asyncMessagesSince(const Chat &chat, const QDate &date)
+{
+	return QtConcurrent::run(this, &HistorySqlStorage::syncMessagesSince, chat, date);
+}
+
+QVector<Message> HistorySqlStorage::getMessagesBackTo(const Chat &chat, const QDateTime &datetime, int limit)
 {
 	DatabaseMutex.lock();
 
-	QList<Message> result;
+	QVector<Message> result;
 
 	QSqlQuery query(Database);
 	// we want last *limit* messages, so we have to invert sorting here
 	// it is reverted back manually below
-	QString queryString = "SELECT chat, sender, content, send_time, receive_time, attributes FROM kadu_messages WHERE " + chatWhere(chat) +
-			" AND receive_time >= :date ORDER BY receive_time DESC, rowid DESC LIMIT :limit";
+	QString queryString = "SELECT chat.uuid, con.uuid, kmc.content, send_time, receive_time, is_outgoing FROM kadu_messages "
+			"LEFT JOIN kadu_chats chat ON (kadu_messages.chat_id=chat.id) "
+			"LEFT JOIN kadu_contacts con ON (kadu_messages.contact_id=con.id) "
+			"LEFT JOIN kadu_message_contents kmc ON (kadu_messages.content_id=kmc.id) WHERE " + chatWhere(chat) +
+			" AND receive_time >= :datetime ORDER BY date_id DESC, kadu_messages.rowid DESC LIMIT :limit";
 	query.prepare(queryString);
 
-	query.bindValue(":chat", chat.uuid().toString());
-	query.bindValue(":date", datetime.toString(Qt::ISODate));
+	query.bindValue(":datetime", datetime.toString(Qt::ISODate));
 	query.bindValue(":limit", limit);
 
 	executeQuery(query);
@@ -604,8 +746,8 @@ QList<Message> HistorySqlStorage::messagesBackTo(const Chat &chat, const QDateTi
 
 	DatabaseMutex.unlock();
 
-	// se comment above
-	QList<Message> inverted;
+	// see comment above
+	QVector<Message> inverted;
 	inverted.reserve(result.size());
 
 	for (int i = result.size() - 1; i >= 0; --i)
@@ -613,10 +755,26 @@ QList<Message> HistorySqlStorage::messagesBackTo(const Chat &chat, const QDateTi
 	return inverted;
 }
 
+QVector<Message> HistorySqlStorage::syncGetMessagesBackTo(const Chat &chat, const QDateTime &datetime, int limit)
+{
+	if (!isDatabaseReady(true))
+		return QVector<Message>();
+
+	return getMessagesBackTo(chat, datetime, limit);
+}
+
+QFuture<QVector<Message> > HistorySqlStorage::asyncMessagesBackTo(const Chat &chat, const QDateTime &datetime, int limit)
+{
+	return QtConcurrent::run(this, &HistorySqlStorage::syncGetMessagesBackTo, chat, datetime, limit);
+}
+
 QList<QString> HistorySqlStorage::smsRecipientsList(const HistorySearchParameters &search)
 {
 	kdebugf();
 
+	if (!isDatabaseReady(false))
+		return QList<QString>();
+
 	QMutexLocker locker(&DatabaseMutex);
 
 	QSqlQuery query(Database);
@@ -648,18 +806,21 @@ QList<QString> HistorySqlStorage::smsRecipientsList(const HistorySearchParameter
 	return recipients;
 }
 
-QList<DatesModelItem> HistorySqlStorage::datesForSmsRecipient(const QString &recipient, const HistorySearchParameters &search)
+QVector<DatesModelItem> HistorySqlStorage::datesForSmsRecipient(const QString &recipient, const HistorySearchParameters &search)
 {
 	kdebugf();
 
 	if (recipient.isEmpty())
-		return QList<DatesModelItem>();
+		return QVector<DatesModelItem>();
+
+	if (!isDatabaseReady(false))
+		return QVector<DatesModelItem>();
 
 	QMutexLocker locker(&DatabaseMutex);
 
 	QSqlQuery query(Database);
 	QString queryString = "SELECT count(1), substr(send_time,0,11)";
-	queryString += " FROM (select send_time from kadu_sms where receipient = :receipient";
+	queryString += " FROM (SELECT send_time FROM kadu_sms WHERE receipient = :receipient";
 
 	if (!search.query().isEmpty())
 		queryString += " AND content LIKE :content";
@@ -681,7 +842,7 @@ QList<DatesModelItem> HistorySqlStorage::datesForSmsRecipient(const QString &rec
 	if (search.toDate().isValid())
 		query.bindValue(":toDate", search.toDate());
 
-	QList<DatesModelItem> dates;
+	QVector<DatesModelItem> dates;
 	executeQuery(query);
 
 	while (query.next())
@@ -690,16 +851,19 @@ QList<DatesModelItem> HistorySqlStorage::datesForSmsRecipient(const QString &rec
 		if (!date.isValid())
 			continue;
 
-		dates.append(DatesModelItem(date, query.value(0).toInt(), QString()));
+		dates.append(DatesModelItem(date, QString(), query.value(0).toInt()));
 	}
 
 	return dates;
 }
 
-QList<Message> HistorySqlStorage::sms(const QString &recipient, const QDate &date, int limit)
+QVector<Message> HistorySqlStorage::sms(const QString &recipient, const QDate &date, int limit)
 {
 	kdebugf();
 
+	if (!isDatabaseReady(false))
+		return QVector<Message>();
+
 	QMutexLocker locker(&DatabaseMutex);
 
 	QSqlQuery query(Database);
@@ -719,15 +883,18 @@ QList<Message> HistorySqlStorage::sms(const QString &recipient, const QDate &dat
 		query.bindValue(":limit", limit);
 	executeQuery(query);
 
-	QList<Message> result = smsFromQuery(query);
+	QVector<Message> result = smsFromQuery(query);
 
 	return result;
 }
 
-QList<Buddy> HistorySqlStorage::statusBuddiesList(const HistorySearchParameters &search)
+QVector<Buddy> HistorySqlStorage::statusBuddiesList(const HistorySearchParameters &search)
 {
 	kdebugf();
 
+	if (!isDatabaseReady(false))
+		return QVector<Buddy>();
+
 	QMutexLocker locker(&DatabaseMutex);
 
 	QSqlQuery query(Database);
@@ -749,37 +916,39 @@ QList<Buddy> HistorySqlStorage::statusBuddiesList(const HistorySearchParameters
 	if (search.toDate().isValid())
 		query.bindValue(":toDate", search.toDate());
 
-	QList<Buddy> buddies;
-	QSet<Contact> usedContacts;
+	QVector<Buddy> buddies;
 
 	executeQuery(query);
 	while (query.next())
 	{
 		Contact contact = ContactManager::instance()->byUuid(query.value(0).toString());
-		if (contact && !usedContacts.contains(contact))
-		{
-			Buddy buddy = BuddyManager::instance()->byContact(contact, ActionCreateAndAdd);
+		if (!contact)
+			continue;
+
+		Buddy buddy = BuddyManager::instance()->byContact(contact, ActionCreateAndAdd);
+		Q_ASSERT(buddy);
+		if (!buddies.contains(buddy))
 			buddies.append(buddy);
-			foreach (const Contact &contact, buddy.contacts())
-				usedContacts.insert(contact);
-		}
 	}
 
 	return buddies;
 }
 
-QList<DatesModelItem> HistorySqlStorage::datesForStatusBuddy(const Buddy &buddy, const HistorySearchParameters &search)
+QVector<DatesModelItem> HistorySqlStorage::datesForStatusBuddy(const Buddy &buddy, const HistorySearchParameters &search)
 {
 	kdebugf();
 
 	if (!buddy)
-		return QList<DatesModelItem>();
+		return QVector<DatesModelItem>();
+
+	if (!isDatabaseReady(false))
+		return QVector<DatesModelItem>();
 
 	QMutexLocker locker(&DatabaseMutex);
 
 	QSqlQuery query(Database);
 	QString queryString = "SELECT count(1), substr(set_time,0,11) FROM";
-	queryString += " (select set_time FROM kadu_statuses WHERE " + buddyContactsWhere(buddy);
+	queryString += " (SELECT set_time FROM kadu_statuses WHERE " + buddyContactsWhere(buddy, "contact");
 
 	if (!search.query().isEmpty())
 		queryString += " AND description LIKE :description";
@@ -788,8 +957,8 @@ QList<DatesModelItem> HistorySqlStorage::datesForStatusBuddy(const Buddy &buddy,
 	if (search.toDate().isValid())
 		queryString += " AND substr(set_time,0,11) <= :toDate";
 
-	queryString += "order by set_time DESC, rowid DESC)";
-	queryString += "group by substr(set_time,0,11) order by set_time ASC";
+	queryString += " ORDER BY set_time DESC, rowid DESC)";
+	queryString += " GROUP BY substr(set_time,0,11) ORDER BY set_time ASC";
 
 	query.prepare(queryString);
 
@@ -800,7 +969,7 @@ QList<DatesModelItem> HistorySqlStorage::datesForStatusBuddy(const Buddy &buddy,
 	if (search.toDate().isValid())
 		query.bindValue(":toDate", search.toDate());
 
-	QList<DatesModelItem> dates;
+	QVector<DatesModelItem> dates;
 
 	executeQuery(query);
 
@@ -811,7 +980,7 @@ QList<DatesModelItem> HistorySqlStorage::datesForStatusBuddy(const Buddy &buddy,
 		if (!date.isValid())
 			continue;
 
-		dates.append(DatesModelItem(date, query.value(0).toInt(), QString()));
+		dates.append(DatesModelItem(date, QString(), query.value(0).toInt()));
 	}
 
 	return dates;
@@ -821,10 +990,13 @@ QList<TimedStatus> HistorySqlStorage::statuses(const Buddy &buddy, const QDate &
 {
 	kdebugf();
 
+	if (!isDatabaseReady(false))
+		return QList<TimedStatus>();
+
 	QMutexLocker locker(&DatabaseMutex);
 
 	QSqlQuery query(Database);
-	QString queryString = "SELECT contact, status, description, set_time FROM kadu_statuses WHERE " + buddyContactsWhere(buddy);
+	QString queryString = "SELECT contact, status, description, set_time FROM kadu_statuses WHERE " + buddyContactsWhere(buddy, "contact");
 	if (!date.isNull())
 		queryString += " AND substr(set_time,0,11) = :date";
 	queryString += " ORDER BY set_time ASC";
@@ -851,27 +1023,41 @@ void HistorySqlStorage::executeQuery(QSqlQuery &query)
 
 	query.setForwardOnly(true);
 
-// 	QDateTime before = QDateTime::currentDateTime();
+	QDateTime before = QDateTime::currentDateTime();
 	query.exec();
-// 	QDateTime after = QDateTime::currentDateTime();
-
-// 	printf("[%s]\n[%d.%d]-[%d.%d]/%d.%d\n", qPrintable(query.executedQuery()),
-// 			before.toTime_t(), before.time().msec(),
-// 			after.toTime_t(), after.time().msec(),
-// 			after.toTime_t() - before.toTime_t(),
-// 			after.time().msec() - before.time().msec());
-
-
+	QDateTime after = QDateTime::currentDateTime();
 	kdebugm(KDEBUG_INFO, "db query: %s\n", qPrintable(query.executedQuery()));
+
+/*
+	printf("[%s]\n[%d.%d]-[%d.%d]/%d.%d\n", qPrintable(query.executedQuery()),
+			before.toTime_t(), before.time().msec(),
+			after.toTime_t(), after.time().msec(),
+			after.toTime_t() - before.toTime_t(),
+			after.time().msec() - before.time().msec());
+*/
 }
 
-
-QList<Message> HistorySqlStorage::messagesFromQuery(QSqlQuery &query)
+QString HistorySqlStorage::stripAllScriptTags(const QString &string)
 {
-	QList<Message> messages;
+	QString beforeReplace = string;
+	QString afterReplace = beforeReplace;
+
+	afterReplace.replace("<script", "", Qt::CaseInsensitive);
+	while (beforeReplace != afterReplace)
+	{
+		beforeReplace = afterReplace;
+		afterReplace.replace("<script", "", Qt::CaseInsensitive);
+	}
+
+	return afterReplace;
+}
+
+QVector<Message> HistorySqlStorage::messagesFromQuery(QSqlQuery &query)
+{
+	QVector<Message> messages;
 	while (query.next())
 	{
-		bool outgoing = QVariant(query.value(5).toString().split('=').last()).toBool();
+		bool outgoing = query.value(5).toBool();
 
 		Chat chat = ChatManager::instance()->byUuid(query.value(0).toString());
 		if (chat.isNull())
@@ -888,7 +1074,7 @@ QList<Message> HistorySqlStorage::messagesFromQuery(QSqlQuery &query)
 		message.setMessageChat(chat);
 		message.setType(type);
 		message.setMessageSender(sender);
-		message.setContent(query.value(2).toString());
+		message.setContent(stripAllScriptTags(query.value(2).toString()));
 		message.setSendDate(query.value(3).toDateTime());
 		message.setReceiveDate(query.value(4).toDateTime());
 		message.setStatus(outgoing ? MessageStatusDelivered : MessageStatusReceived);
@@ -911,8 +1097,8 @@ QList<TimedStatus> HistorySqlStorage::statusesFromQuery(QSqlQuery &query)
 			continue;
 
 		Status status;
-		status.setType(query.value(1).toString());
-		status.setDescription(query.value(2).toString());
+		status.setType(StatusTypeManager::instance()->fromName(query.value(1).toString()));
+		status.setDescription(Qt::escape(query.value(2).toString()));
 
 		TimedStatus timedStatus(status, query.value(3).toDateTime());
 
@@ -922,9 +1108,9 @@ QList<TimedStatus> HistorySqlStorage::statusesFromQuery(QSqlQuery &query)
 	return statuses;
 }
 
-QList<Message> HistorySqlStorage::smsFromQuery(QSqlQuery &query)
+QVector<Message> HistorySqlStorage::smsFromQuery(QSqlQuery &query)
 {
-	QList<Message> messages;
+	QVector<Message> messages;
 
 	while (query.next())
 	{
@@ -933,7 +1119,7 @@ QList<Message> HistorySqlStorage::smsFromQuery(QSqlQuery &query)
 		message.setType(MessageTypeSystem);
 		message.setReceiveDate(query.value(1).toDateTime());
 		message.setSendDate(query.value(1).toDateTime());
-		message.setContent(query.value(0).toString());
+		message.setContent(Qt::escape(query.value(0).toString()));
 
 		messages.append(message);
 	}
