@@ -1,9 +1,9 @@
 /*
  * %kadu copyright begin%
- * Copyright 2010 Bartosz Brachaczek (b.brachaczek@gmail.com)
  * Copyright 2009, 2010, 2011 Piotr Galiszewski (piotr.galiszewski@kadu.im)
- * Copyright 2009 Wojciech Treter (juzefwt@gmail.com)
- * Copyright 2009, 2010 Rafał Malinowski (rafal.przemyslaw.malinowski@gmail.com)
+ * Copyright 2009, 2009, 2009, 2009 Wojciech Treter (juzefwt@gmail.com)
+ * Copyright 2009, 2010, 2011 Rafał Malinowski (rafal.przemyslaw.malinowski@gmail.com)
+ * Copyright 2010, 2011 Bartosz Brachaczek (b.brachaczek@gmail.com)
  * %kadu copyright end%
  *
  * This program is free software; you can redistribute it and/or
@@ -29,6 +29,10 @@
 
 #include "plugins/history/storage/history-storage.h"
 
+class QSqlError;
+
+class ProgressWindow2;
+
 /**
 	@class HistorySqlStorage
 	@author Juzef, Adrian
@@ -38,53 +42,74 @@ class HistorySqlStorage : public HistoryStorage
 {
 	Q_OBJECT
 
+	QThread *InitializerThread;
+	ProgressWindow2 *ImportProgressWindow;
+
 	QSqlDatabase Database;
 
-	QSqlQuery ListChatsQuery;
 	QSqlQuery AppendMessageQuery;
 	QSqlQuery AppendStatusQuery;
 	QSqlQuery AppendSmsQuery;
 
 	QMutex DatabaseMutex;
 
-	void initDatabase();
-	void initQueries();
-	void initTables();
-	void initIndexes();
-	void initKaduMessagesTable();
-	void initKaduStatusesTable();
-	void initKaduSmsTable();
+	QMap<Chat, int> ChatMap;
+	QMap<Contact, int> ContactMap;
+	QMap<QString, int> DateMap;
 
-	QString chatWhere(const Chat &chat);
-	QString buddyContactsWhere(const Buddy &buddy);
+	void initQueries();
+
+	int findOrCreateChat(const Chat &chat);
+	int findOrCreateContact(const Contact &contact);
+	int saveMessageContent(const Message &message);
+	int findOrCreateDate(const QDate &date);
+
+	QString chatWhere(const Chat &chat, const QString &chatPrefix = "chat.");
+	QString buddyContactsWhere(const Buddy &buddy, const QString &fieldName);
+
+	static QString stripAllScriptTags(const QString &string);
 
 	void executeQuery(QSqlQuery &query);
-	QList<Message> messagesFromQuery(QSqlQuery &query);
+	QVector<Message> messagesFromQuery(QSqlQuery &query);
 	QList<TimedStatus> statusesFromQuery(QSqlQuery &query);
-	QList<Message> smsFromQuery(QSqlQuery &query);
+	QVector<Message> smsFromQuery(QSqlQuery &query);
+
+	bool isDatabaseReady(bool wait);
+
+	QVector<Message> getMessagesSince(const Chat &chat, const QDate &date);
+	QVector<Message> syncMessagesSince(const Chat &chat, const QDate &date);
+
+	QVector<Message> getMessagesBackTo(const Chat &chat, const QDateTime &datetime, int limit);
+	QVector<Message> syncGetMessagesBackTo(const Chat &chat, const QDateTime &datetime, int limit);
 
 private slots:
 	virtual void messageReceived(const Message &message);
 	virtual void messageSent(const Message &message);
 
+	void databaseReady(bool ok);
+
+	void importStarted();
+	void importFinished();
+	void databaseOpenFailed(const QSqlError &error);
+
 public:
 	explicit HistorySqlStorage(QObject *parent = 0);
 	virtual ~HistorySqlStorage();
 
-	virtual QList<Chat> chats(const HistorySearchParameters &search);
+	virtual QVector<Chat> chats(const HistorySearchParameters &search);
 
-	virtual QList<DatesModelItem> chatDates(const Chat &chat, const HistorySearchParameters &search);
-	virtual QList<Message> messages(const Chat &chat, const QDate &date = QDate(), int limit = 0);
-	virtual QList<Message> messagesSince(const Chat &chat, const QDate &date);
-	virtual QList<Message> messagesBackTo(const Chat &chat, const QDateTime &datetime, int limit);
+	virtual QVector<DatesModelItem> chatDates(const Chat &chat, const HistorySearchParameters &search);
+	virtual QVector<Message> messages(const Chat &chat, const QDate &date = QDate(), int limit = 0);
+	virtual QFuture<QVector<Message> > asyncMessagesSince(const Chat &chat, const QDate &date);
+	virtual QFuture<QVector<Message> > asyncMessagesBackTo(const Chat &chat, const QDateTime &datetime, int limit);
 
-	virtual QList<Buddy> statusBuddiesList(const HistorySearchParameters &search);
-	virtual QList<DatesModelItem> datesForStatusBuddy(const Buddy &buddy, const HistorySearchParameters &search);
+	virtual QVector<Buddy> statusBuddiesList(const HistorySearchParameters &search);
+	virtual QVector<DatesModelItem> datesForStatusBuddy(const Buddy &buddy, const HistorySearchParameters &search);
 	virtual QList<TimedStatus> statuses(const Buddy &buddy, const QDate &date = QDate(), int limit = 0);
 
 	virtual QList<QString> smsRecipientsList(const HistorySearchParameters &search);
-	virtual QList<DatesModelItem> datesForSmsRecipient(const QString &recipient, const HistorySearchParameters &search);
-	virtual QList<Message> sms(const QString &recipient, const QDate &date = QDate(), int limit = 0);
+	virtual QVector<DatesModelItem> datesForSmsRecipient(const QString &recipient, const HistorySearchParameters &search);
+	virtual QVector<Message> sms(const QString &recipient, const QDate &date = QDate(), int limit = 0);
 
 	virtual void appendMessage(const Message &message);
 	virtual void appendStatus(const Contact &contact, const Status &status, const QDateTime &time = QDateTime::currentDateTime());
